""" Matchmaker, Matchmaker, make me a fetcher """

from typing import List, Union

# pylint: disable=E0401, E0402, E1136, R0913

from .fetch_plan import ParallelFetchPlan
from .fetchers import (
    NgasStreamingFileFetcher,
    NgasDirectCopyFileFetcher,
    NgasModeDetectingFetcher,
    RetryableFileFetcher,
    OracleXmlFetcher,
    DryRunFakeFileFetcher,
    ProgressReportingFetcher,
)
from .interfaces import FetcherFactory, FileFetcher, FetchPlan
from .locations import NgasFile, OracleXml
from .progress import ProgressBarReporter
from .validators import SizeValidator, Crc32Validator, XmlWellFormedValidator


class ConfiguredFetcherFactory(FetcherFactory):
    """
    A generic FetcherFactory which is configured by some arguments originating probably
    in the CLI arguments. Pass this through the location report to get a fetch plan.
    """

    def __init__(
        self,
        concurrency=1,
        streaming=False,
        direct_copy=False,
        dry_run=False,
        force=False,
        aggregate_size=0,
    ):
        # we only have one kind of OracleXmlFetcher
        self.xml_fetcher = OracleXmlFetcher

        # in dry run mode, use fake fetchers
        if dry_run:
            self.file_fetcher = DryRunFakeFileFetcher
            self.xml_fetcher = DryRunFakeFileFetcher
        elif streaming:
            self.file_fetcher = NgasStreamingFileFetcher
        elif direct_copy:
            self.file_fetcher = NgasDirectCopyFileFetcher
        else:
            self.file_fetcher = NgasModeDetectingFetcher

        self.concurrency = concurrency
        self.force = force
        self.progressbar = ProgressBarReporter(aggregate_size)

    def fetch_ngas(self, file: NgasFile) -> FileFetcher:
        """
        Fetch a file from NGAS, retrying a few times in the event of error

        :param file: the file to fetch
        :return:
        """
        return ProgressReportingFetcher(
            self.progressbar,
            RetryableFileFetcher(self.file_fetcher(file, force=self.force))
            .and_validate_with(SizeValidator(file.size))
            .and_validate_with(Crc32Validator(file.checksum)),
        )

    def fetch_oracle_xml(self, file: OracleXml) -> FileFetcher:
        """
        Fetch an SDM table from the ALMA DB, retrying a few times in the event of error

        :param file:
        :return:
        """
        return ProgressReportingFetcher(
            self.progressbar,
            RetryableFileFetcher(self.xml_fetcher(file, force=self.force))
            .and_validate_with(SizeValidator(file.size))
            .and_validate_with(XmlWellFormedValidator()),
        )

    def fetch_plan(
        self, fetchers: List[Union[FileFetcher, FetchPlan]], concurrency: int = None
    ) -> FetchPlan:
        """
        Decide whether or not to fetch in parallel.

        :param fetchers: fetchers and/or fetch plans
        :param concurrency: number of concurrent jobs permitted
        :return:
        """
        if not concurrency:
            concurrency = self.concurrency
        # Take whichever is smaller of my permitted concurrency and the requested concurrency
        return ParallelFetchPlan(fetchers, concurrency=min(concurrency, self.concurrency))
