import glob
import logging
import os
import re
import sys

from vela.forger import forge
from vela.interfaces import CasaEmulatorIF


"""
Emulate a CASA run
"""


def get_base_environment(parameters: dict):
    os.environ["SCIPIPE_ROOTDIR"] = parameters["rootDirectory"]
    os.environ["CASA_HOME"] = parameters["homeForReprocessing"]
    os.environ["LANG"] = "en_US.UTF-8"


def check_processing_env(logger: logging.Logger):
    logger.info("Checking processing environment:")
    env1 = os.environ.get("SCIPIPE_ROOTDIR")
    logger.info(f"SCIPIPE_ROOTDIR: {env1}")
    env2 = os.environ.get("CASA_HOME")
    logger.info(f"CASA_HOME: {env2}")
    env3 = os.environ.get("PPR_FILENAME")
    logger.info(f"PPR_FILENAME: {env3}")
    env4 = os.environ.get("LANG")
    logger.info(f"LANG: {env4}")
    if "None" in [env1, env2, env3, env4]:
        logger.info("Environment setup Failed!")
        sys.exit(1)
    else:
        logger.info("Environment ready for processing")


class CalibrationEmulator(CasaEmulatorIF):
    def __init__(self, metadata: str, ppr: str):
        self.logger = logging.getLogger("vela")
        self.ppr = ppr
        self.metadata = metadata

    def run(self):
        self.logger.info("Vela waiting on the Forger....")
        forge(self.metadata)

    def setup_environment(self, parameters: dict):
        get_base_environment(parameters)
        os.environ["PPR_FILENAME"] = str(self.ppr)

        check_processing_env(self.logger)

    def check_logs(self, parent_path: str):
        self.logger.info("CHECKING VELA LOGS!")

        if not os.getcwd().endswith("/working"):
            os.chdir(parent_path + "/working")

        vela_logs = glob.glob("vela-*.log")

        for file in vela_logs:
            if re.match("^.*SEVERE\sflagmanager.*$", open(file).read()):
                self.logger.info("VELA ERROR! Please inspect logs.")
            else:
                self.logger.info("VELA Success!")


class ImagingEmulator(CasaEmulatorIF):
    def __init__(self, metadata: str, ppr: str):
        self.logger = logging.getLogger("vela")
        self.metadata = metadata
        self.ppr = ppr

    def run(self):
        self.logger.info("Vela waiting on the Forger....")
        forge(self.metadata)

    def setup_environment(self, parameters: dict):
        get_base_environment(parameters)
        os.environ["PPR_FILENAME"] = str(self.ppr)

        check_processing_env(self.logger)

    def check_logs(self, parent_path: str):
        self.logger.info("CHECKING VELA LOGS!")

        if not os.getcwd().endswith("/working"):
            os.chdir(parent_path + "/working")

        vela_logs = glob.glob("vela-*.log")

        for file in vela_logs:
            if re.match("^.*SEVERE\sflagmanager.*$", open(file).read()):
                self.logger.info("VELA ERROR! Please inspect logs.")
            else:
                self.logger.info("VELA Success!")
