"""
Forger has two tasks: faking CASA logs and faking CASA products
"""
import logging
import os

import json
import tarfile
from pathlib import Path

import pendulum.date
import vela.content.forger_content as content


class VelaLog:
    def __init__(self, metadata: str):
        self.logger = logging.getLogger("vela")
        self.metadata = metadata

    def forge_logs(self, parameters: dict):
        # forging three files. 2 for pipeline completion, 1 log file
        # vela-pipeline.sh.out.txt
        # vela-pipeline.sh.err.txt
        # vela-<date_as_string>-<timestring>.log

        workflow_name = parameters["workflowName"]

        if "calibration" in workflow_name:
            self.cal_logs()
        elif "imaging" in workflow_name:
            self.img_logs()

    def cal_logs(self):
        path = Path.cwd() / "working"
        os.chdir(path)

        # make log files
        pipe_out = open("vela-pipeline.sh.out.txt", "x")
        pipe_out.write(forged_content())
        pipe_out.close()

        pipe_err = open("vela-pipeline.sh.err.txt", "x")
        pipe_err.write(forged_content())
        pipe_err.close()

        datestring = pendulum.now().format("YYYYMMDD")
        timestring = pendulum.now().utcnow().format("hhmmss")

        log = open(f"vela-{datestring}-{timestring}.log", "w")
        log.write(forged_content())
        log.close()

        self.logger.info("Logs forging complete.")
        os.chdir("../")

    def img_logs(self):
        # TODO: find out if logs are different for imaging
        self.cal_logs()


class VelaProduct:
    def __init__(self, metadata: str):
        self.logger = logging.getLogger("vela")
        self.metadata = metadata

    def forge_products(self, parameters: dict):
        workflow_name = parameters["workflowName"]

        if "calibration" in workflow_name:
            self.cal_products(parameters)
        elif "imaging" in workflow_name:
            self.img_products(parameters)

    def cal_products(self, parameters: dict):
        # forge products so there is something to test delivery with

        path = Path.cwd() / "products"
        os.chdir(path)

        sdm_id = parameters["fileSetIds"]
        filename = sdm_id + ".ms.calapply.txt"

        calapply = open(filename, "x")
        calapply.write("I am a calibration file.")
        calapply.close()

        os.chdir("../working")
        self.forge_measurement_set(sdm_id)
        self.forge_weblog(os.getcwd())

        self.logger.info("Forging calibration products complete.")
        os.chdir("../")

    def img_products(self, parameters: dict):
        path = Path.cwd() / "products"
        os.chdir(path)

        filename = "oussid.J1106-3646_sci.X.pb.tt0.fits"

        fits = open(filename, "x")
        fits.write("I am an image.")
        fits.close()

        os.chdir("../working")
        self.forge_weblog(os.getcwd())

        self.logger.info("Forging image products complete.")
        os.chdir("../")

    def forge_measurement_set(self, sdm_id: str):
        self.logger.info("Forging Measurement Set....")
        cms_name = sdm_id + ".ms"
        os.mkdir(cms_name)
        os.chdir("./" + cms_name)

        for i in range(0, 5):
            dirname = f"FORGED_MS_DIR_{i}"
            os.mkdir(dirname)
            file = open(f"table.f{i}", "x")
            file.write("I am a table.")
            file.close()

        os.chdir("../")
        self.logger.info("MS forging complete.")

    def forge_weblog(self, path):
        self.logger.info("Forging weblog...")

        datestring = pendulum.now().format("YYYYMMDD")
        timestring = pendulum.now().utcnow().format("hhmmss")
        dirname = f"pipeline-{datestring}-{timestring}"
        os.mkdir(dirname)
        os.chdir(path + "/" + dirname)
        os.mkdir("html")
        os.chdir(path + "/" + dirname + "/html")

        file1 = open(f"vela-{datestring}-{timestring}.log", "x")
        file1.write("I am a weblog file")
        file1.close()

        file2 = open("index.html", "x")
        file2.write("I am a weblog file")
        file2.close()

        # qa notes are not created by casa, blank file is created in conveyor at delivery
        # file3 = open("qa_notes.html", "x")
        # file3.write("I am QA Notes")
        # file3.close()

        os.chdir("../../../products")

        with tarfile.open("weblog.tgz", "w:gz") as tar:
            tar.add(path + "/" + dirname, arcname=os.path.basename(dirname))
            tar.close()

        self.logger.info("Weblog forging complete.")


def read_metadata(filename: str) -> dict:
    with open(filename) as file:
        return json.loads(file.read())


def forged_content() -> str:
    return content.file_content()


# write fake logs and products
def forge(metadata: str):
    parameters = read_metadata(metadata)
    fake_logs = VelaLog(metadata)
    fake_logs.forge_logs(parameters)

    fake_products = VelaProduct(metadata)
    fake_products.forge_products(parameters)
